package com.persagy.adm.diagram.controller;

import com.fasterxml.jackson.databind.node.ObjectNode;
import com.persagy.adm.diagram.core.DataStrategy;
import com.persagy.adm.diagram.core.model.Diagram;
import com.persagy.adm.diagram.core.model.template.DiagramTemplate;
import com.persagy.dmp.common.model.response.CommonResult;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.web.bind.annotation.*;

import java.util.List;

/**
 * 系统图相关接口
 *
 * @author liyang
 * @date 2022-01-04
 */
@RestController
@CrossOrigin
@RequestMapping("/diagram")
public class DiagramController {

    @Autowired
    @Qualifier("dataStrategyImpl")
    private DataStrategy dataStrategy;

    /**
     * 系统图列表接口
     * 根据系统图类型、项目id、系统实例id、集团code、系统图名称查询
     *
     * @param projectId 项目id
     * @param systemId  系统实例id
     * @param groupCode 集团code
     * @param name      系统图名称
     * @return 系统图列表
     */
    @RequestMapping("/getDiagrams")
    public CommonResult<List<Diagram>> getDiagrams(@RequestParam String projectId,
                                                   @RequestParam(required = false) String systemId,
                                                   @RequestParam(required = false) String groupCode,
                                                   @RequestParam(required = false) String name) {
        return CommonResult.success(dataStrategy.getDiagrams(null, projectId, systemId,
                groupCode, name));
    }

    /**
     * 新增系统图
     *
     * @param diagram 系统图对象信息
     * @return 是否成功
     */
    @PostMapping("/newDiagram")
    public CommonResult<Diagram> newDiagram(@RequestBody Diagram diagram) {
        return CommonResult.success(dataStrategy.saveDiagram(diagram));
    }

    /**
     * 系统图删除功能
     *
     * @param diagramId 系统图id
     * @return 是否成功
     */
    @GetMapping("/deleteDiagram")
    public CommonResult<Boolean> deleteDiagram(String diagramId) {
        return CommonResult.success(dataStrategy.deleteDiagram(diagramId));
    }

    /**
     * 根据系统图id获取系统图信息
     *
     * @param diagramId 系统图id
     * @return 系统图对象信息
     */
    @GetMapping("/getDiagram")
    public CommonResult<Diagram> getDiagram(String diagramId) {
        return CommonResult.success(dataStrategy.getDiagram(diagramId));
    }

    /**
     * 系统图实例查询
     *
     * @param projectId 项目id
     * @param groupCode 集团code
     * @return 系统实例
     */
    @GetMapping("/system/instance")
    public CommonResult<List<ObjectNode>> loadSystemInstance(String projectId, String groupCode) {
        return CommonResult.success(dataStrategy.loadSystemInstance(projectId, groupCode));
    }

    /**
     * 根据系统图类型获取模板列表
     *
     * @param diagramType 系统图类型
     * @return 模板列表
     */
    @GetMapping("/getTemplates/ByType")
    public CommonResult<List<DiagramTemplate>> getTemplatesByType(String diagramType) {
        return CommonResult.success(dataStrategy.getTemplates(diagramType));
    }

    /**
     * 根据模板id获取系统模板信息
     *
     * @param templateId 模板id
     * @return 模板信息
     */
    @GetMapping("/getTemplate/ById")
    public CommonResult<DiagramTemplate> getTemplateById(String templateId) {
        return CommonResult.success(dataStrategy.getTemplate(templateId));
    }

    /**
     * 获取所有的系统模板
     *
     * @return 模板列表
     */
    @GetMapping("/getTemplates")
    public CommonResult<List<DiagramTemplate>> getTemplates() {
        return CommonResult.success(dataStrategy.getTemplates());
    }
}
