package com.persagy.adm.diagram.core.impl;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.util.StrUtil;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.fasterxml.jackson.databind.node.ArrayNode;
import com.fasterxml.jackson.databind.node.JsonNodeFactory;
import com.fasterxml.jackson.databind.node.ObjectNode;
import com.persagy.adm.diagram.core.DataStrategy;
import com.persagy.adm.diagram.core.model.Diagram;
import com.persagy.adm.diagram.core.model.legend.Legend;
import com.persagy.adm.diagram.core.model.template.DiagramTemplate;
import com.persagy.adm.diagram.dao.DiagramMapper;
import com.persagy.adm.diagram.dao.DiagramTypeMapper;
import com.persagy.adm.diagram.dao.LegendMapper;
import com.persagy.adm.diagram.dao.TemplateMapper;
import com.persagy.adm.diagram.entity.*;
import com.persagy.adm.diagram.util.ServiceUtil;
import com.persagy.dmp.basic.model.QueryCriteria;
import com.persagy.dmp.basic.model.QueryOperator;
import com.persagy.dmp.basic.utils.JsonNodeUtils;
import com.persagy.dmp.common.constant.ResponseCode;
import com.persagy.dmp.common.exception.BusinessException;
import com.persagy.dmp.digital.client.DigitalObjectFacade;
import com.persagy.dmp.digital.client.DigitalRelationFacade;
import com.persagy.dmp.digital.entity.ObjectRelation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.stereotype.Service;

import java.util.*;
import java.util.concurrent.CompletableFuture;
import java.util.concurrent.ThreadPoolExecutor;
import java.util.stream.Collectors;

/**
 * 相关接口实现
 *
 * @author liyang
 * @date 2021-12-31
 */
@Service("dataStrategyImpl")
public class DataStrategyImpl implements DataStrategy {

    @Autowired
    private ModelAdapter modelAdapter;
    @Autowired
    private ObjectMapper objectMapper;
    @Autowired
    private DiagramTypeMapper diagramTypeMapper;
    @Autowired
    private TemplateMapper templateMapper;
    @Autowired
    private LegendMapper legendMapper;
    @Autowired
    private DiagramMapper diagramMapper;
    @Autowired
    @Qualifier("customThreadPool")
    private ThreadPoolExecutor executor;

    /**
     * 获取系统图类型列表
     *
     * @return 系统图类型列表
     */
    @Override
    public List<DiagramType> getDiagramTypes() {
        return diagramTypeMapper.getDiagramTypes();
    }

    /**
     * 获取系统图模板列表
     *
     * @return 模板列表
     */
    @Override
    public List<DiagramTemplate> getTemplates() {
        List<TemplateEntity> templates = templateMapper.getTemplates();
        if (CollectionUtil.isNotEmpty(templates)) {
            return templates.stream().map(templateEntity ->
                    modelAdapter.toTemplate(templateEntity)).collect(Collectors.toList());
        }
        return new ArrayList<>();
    }

    /**
     * 根据系统图类型获取系统图模板列表
     *
     * @param diagramType 类型
     * @return 模板列表
     */
    @Override
    public List<DiagramTemplate> getTemplates(String diagramType) {
        if (StrUtil.isBlank(diagramType)) {
            throw new BusinessException(ResponseCode.A0400.getCode(), "diagramType不能为空");
        }
        List<TemplateEntity> templates = templateMapper.getTemplatesByType(diagramType);
        if (CollectionUtil.isNotEmpty(templates)) {
            return templates.stream().map(templateEntity ->
                    modelAdapter.toTemplate(templateEntity)).collect(Collectors.toList());
        }
        return new ArrayList<>();
    }

    /**
     * 保存系统图模板
     *
     * @param template 系统图模板信息
     * @return
     */
    @Override
    public DiagramTemplate saveTemplate(DiagramTemplate template) {
        TemplateEntity templateEntity = modelAdapter.toTemplateEntity(template);
        templateMapper.saveTemplate(templateEntity);
        return template;
    }

    /**
     * 根据模板id删除模板
     *
     * @param templateId 模板id
     * @return 是否成功
     */
    @Override
    public boolean deleteTemplate(String templateId) {
        templateMapper.deleteByTemplateId(templateId);
        return true;
    }

    /**
     * 根据模板id获取模板信息
     *
     * @param templateId 模板id
     * @return 系统图模板信息
     */
    @Override
    public DiagramTemplate getTemplate(String templateId) {
        TemplateEntity templateEntity = templateMapper.selectByTemplateId(templateId);
        return templateEntity != null ? modelAdapter.toTemplate(templateEntity) : new DiagramTemplate();
    }

    /**
     * 根据系统code获取图例
     *
     * @param systemCode 所属系统code
     * @return 图例列表
     */
    @Override
    public List<Legend> getLegends(String systemCode) {
        if (StrUtil.isBlank(systemCode)) {
            throw new BusinessException(ResponseCode.A0400.getCode(), "systemCode不能为空");
        }
        List<LegendEntity> legends = legendMapper.getLegends(systemCode);
        if (CollectionUtil.isNotEmpty(legends)) {
            return legends.stream().map(legendEntity ->
                    modelAdapter.toLegend(legendEntity)).collect(Collectors.toList());
        }
        return new ArrayList<>();
    }

    /**
     * 保存图例
     *
     * @param legend     图例信息
     * @param systemCode 所属系统code
     * @return
     */
    @Override
    public Legend saveLegend(Legend legend, String systemCode) {
        legend.setSystem(systemCode);
        legendMapper.saveLegend(modelAdapter.toLegendEntity(legend));
        return legend;
    }

    /**
     * 删除图例
     *
     * @param legendId   图例id
     * @param systemCode 所属系统code
     * @return
     */
    @Override
    public boolean deleteLegend(String legendId, String systemCode) {
        return legendMapper.deleteLegend(legendId, systemCode);
    }

    /**
     * 根据图例id、所属系统code 获取图例信息
     *
     * @param legendId   图例id
     * @param systemCode 所属系统code
     * @return
     */
    @Override
    public Legend getLegend(String legendId, String systemCode) {
        LegendEntity legendEntity = legendMapper.getLegend(legendId, systemCode);
        return legendEntity == null ? new Legend() : modelAdapter.toLegend(legendEntity);
    }

    /**
     * 根据设备类型获取图例列表
     *
     * @param equipmentType 设备类型
     * @return 图例列表
     */
    @Override
    public List<Legend> getLegendsForEquipment(String equipmentType) {
        if (StrUtil.isBlank(equipmentType)) {
            throw new BusinessException(ResponseCode.A0400.getCode(), "equipmentType不能为空");
        }
        String systemCode = Optional.of(equipmentType).map(s -> s.substring(0, 4)).orElse(StrUtil.EMPTY);
        return getLegends(systemCode);
    }

    /**
     * 根据项目id、系统id、集团code 获取系统图列表
     *
     * @param projectId 项目id
     * @param systemId  系统id
     * @param groupCode 集团code
     * @return 系统图列表
     */
    @Override
    public List<Diagram> getDiagrams(String projectId, String systemId, String groupCode) {
        List<DiagramEntity> diagrams = diagramMapper.getDiagrams(null, projectId, systemId, groupCode, null);
        if (CollectionUtil.isNotEmpty(diagrams)) {
            return diagrams.stream().map(diagramEntity ->
                    modelAdapter.toDiagram(diagramEntity)).collect(Collectors.toList());
        }
        return new ArrayList<>();
    }

    /**
     * 根据系统图类型、项目id、系统id、集团code 获取系统图列表
     *
     * @param diagramType 统图类型
     * @param projectId   项目id
     * @param systemId    系统id
     * @param groupCode   集团code
     * @param name        系统图名称
     * @return 系统图列表
     */
    @Override
    public List<Diagram> getDiagrams(String diagramType, String projectId, String systemId, String groupCode, String name) {
        List<DiagramEntity> diagrams = diagramMapper.getDiagrams(diagramType,
                projectId, systemId, groupCode, name);
        if (CollectionUtil.isNotEmpty(diagrams)) {
            return diagrams.stream().map(diagramEntity ->
                    modelAdapter.toDiagram(diagramEntity)).collect(Collectors.toList());
        }
        return new ArrayList<>();
    }

    /**
     * 保存系统图
     *
     * @param diagram 系统图信息
     * @return
     */
    @Override
    public Diagram saveDiagram(Diagram diagram) {
        DiagramEntity diagramEntity = modelAdapter.toDiagramEntity(diagram);
        diagramMapper.saveDiagram(diagramEntity);
        return diagram;
    }

    /**
     * 根据系统图id删除
     *
     * @param diagramId 系统图id
     * @return 是否成功
     */
    @Override
    public boolean deleteDiagram(String diagramId) {
        return diagramMapper.deleteByDiagramId(diagramId);
    }

    /**
     * 根据系统图id获取系统图信息
     *
     * @param diagramId 系统图id
     * @return 系统图信息
     */
    @Override
    public Diagram getDiagram(String diagramId) {
        DiagramEntity diagramEntity = diagramMapper.getDiagram(diagramId);
        return diagramEntity == null ? new Diagram() : modelAdapter.toDiagram(diagramEntity);
    }

    /**
     * 根据设备类型、项目id、系统id、集团code获取对象列表信息
     *
     * @param equipmentTypes 设备类型列表
     * @param projectId      项目id
     * @param systemId       系统id
     * @param groupCode      集团code
     * @return 节点列表
     */
    @Override
    public List<ObjectNode> loadObjectsByType(List<String> equipmentTypes, String projectId,
                                              String systemId, String groupCode) {
        QueryCriteria criteria = ServiceUtil.getQueryCriteria(objectMapper, "equipment");
        if (CollectionUtil.isNotEmpty(equipmentTypes)) {
            ArrayNode types = criteria.getCriteria().putArray("classCode");
            equipmentTypes.forEach(types::add);
        }

        if (StrUtil.isNotBlank(systemId)) {
            ObjectNode on = JsonNodeFactory.instance.objectNode();
            on.put(ObjectRelation.GRAPH_CODE_HUM, "MechSubset");
            on.put(ObjectRelation.REL_CODE_HUM, "Sy2Eq");
            on.put(ObjectRelation.OBJ_FROM_HUM, systemId);
            criteria.getCriteria().set(QueryOperator.RELATION_TO.getIndex(), on);
        }

        return DigitalObjectFacade.query(groupCode, projectId, null, null, criteria);
    }

    /**
     * 根据关系类型列表、节点id列表、项目id、集团code获取对象关系列表
     *
     * @param relationTypes 关系类型列表
     * @param objectIds     节点id列表
     * @param projectId     项目id
     * @param groupCode     集团code
     * @return 关系列表
     */
    @Override
    public List<ObjectNode> loadRelationsByType(List<String[]> relationTypes, List<String> objectIds,
                                                String projectId, String groupCode) {
        if (CollectionUtil.isEmpty(objectIds)) {
            throw new BusinessException(ResponseCode.A0400.getCode(), "objectIds不能为空");
        }
        //查询到的所有关系
        List<ObjectRelation> allRelations = queryRelationsByObjIds(projectId, groupCode, objectIds);
        //所有关系类型
        Set<String> types = new HashSet<>();
        Optional.ofNullable(relationTypes).ifPresent(relations ->
                relations.forEach(strings -> types.addAll(Arrays.asList(strings))));
        Set<String> typeSet = types.stream().filter(Objects::nonNull).collect(Collectors.toSet());
        //最终符合结果的关系
        if (CollectionUtil.isNotEmpty(typeSet)) {
            List<ObjectRelation> res = allRelations.stream().filter(objectRelation ->
                    typeSet.contains(objectRelation.getRelCode())).collect(Collectors.toList());
            //转换并返回
            return JsonNodeUtils.toListNode(res, null, null);
        }
        return JsonNodeUtils.toListNode(allRelations, null, null);
    }

    private List<ObjectRelation> queryRelationsByObjIds(String projectId, String groupCode, List<String> objectIds) {
        List<ObjectRelation> resultList = new ArrayList<>();
        //obj_from
        CompletableFuture<List<ObjectRelation>> fromFuture = CompletableFuture.supplyAsync(() -> {
            QueryCriteria criteria = ServiceUtil.getQueryCriteria(objectMapper);
            ArrayNode objFrom = criteria.getCriteria().putArray("objFrom");
            Optional.ofNullable(objectIds).ifPresent(strings -> strings.forEach(objFrom::add));
            return DigitalRelationFacade.query(groupCode, projectId, null, null, criteria);
        }, executor);
        //obj_to
        CompletableFuture<List<ObjectRelation>> toFuture = CompletableFuture.supplyAsync(() -> {
            QueryCriteria criteria = ServiceUtil.getQueryCriteria(objectMapper);
            ArrayNode objTo = criteria.getCriteria().putArray("objTo");
            Optional.ofNullable(objectIds).ifPresent(strings -> strings.forEach(objTo::add));
            return DigitalRelationFacade.query(groupCode, projectId, null, null, criteria);
        }, executor);

        CompletableFuture.allOf(fromFuture, toFuture).join();

        try {
            resultList.addAll(fromFuture.get());
            resultList.addAll(toFuture.get());
        } catch (Exception e) {
            e.printStackTrace();
        }
        return resultList.stream().distinct().filter(Objects::nonNull).collect(Collectors.toList());
    }

    /**
     * 根据对象id列表获取节点列表
     *
     * @param objectIds 节点id列表
     * @param projectId 项目id
     * @param groupCode 集团code
     * @return 节点列表
     */
    @Override
    public List<ObjectNode> loadObjectsById(List<String> objectIds, String projectId, String groupCode) {
        if (CollectionUtil.isEmpty(objectIds)) {
            throw new BusinessException(ResponseCode.A0400.getCode(), "objectIds不能为空");
        }
        QueryCriteria criteria = ServiceUtil.getQueryCriteria(objectMapper, "equipment");
        ArrayNode arr = criteria.getCriteria().putArray("id");
        objectIds.forEach(arr::add);

        return DigitalObjectFacade.query(groupCode, projectId, null, null, criteria);
    }

    /**
     * 根据关系id列表、项目id、集团code获取关系列表
     *
     * @param relationIds 关系id列表
     * @param projectId   项目id
     * @param groupCode   集团code
     * @return 关系列表
     */
    @Override
    public List<ObjectNode> loadRelationsById(List<String> relationIds, String projectId, String groupCode) {
        if (CollectionUtil.isEmpty(relationIds)) {
            throw new BusinessException(ResponseCode.A0400.getCode(), "relationIds不能为空");
        }
        QueryCriteria criteria = ServiceUtil.getQueryCriteria(objectMapper);
        ArrayNode nodes = criteria.getCriteria().putArray("id");
        relationIds.forEach(nodes::add);

        List<ObjectRelation> result = DigitalRelationFacade.query(groupCode, projectId, null, null, criteria);
        return JsonNodeUtils.toListNode(result, null, null);
    }

    /**
     * 获取系统实例列表
     *
     * @param projectId 项目id
     * @param groupCode 集团code
     * @return 系统实例列表
     */
    @Override
    public List<ObjectNode> loadSystemInstance(String projectId, String groupCode) {
        QueryCriteria criteria = ServiceUtil.getQueryCriteria(objectMapper, "system");
        return DigitalObjectFacade.query(groupCode, projectId, null, null, criteria);
    }
}
